/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the QtDeclarative module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/
#ifndef QMLSCRIPTPARSER_P_H
#define QMLSCRIPTPARSER_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include "qmlerror.h"
#include "qmlparser_p.h"

#include <QtCore/QList>
#include <QtCore/QUrl>

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Declarative)

class QByteArray;

class QmlScriptParserJsASTData;
class QmlScriptParser
{
public:
    class Import
    {
    public:
        Import() : type(Library) {}

        enum Type { Library, File };
        Type type;

        QString uri;
        QString qualifier;
        QString version;

        QmlParser::LocationSpan location;
    };

    class TypeReference
    {
    public:
        TypeReference(int typeId, const QString &typeName) : id(typeId), name(typeName) {}

        int id;
        // type as it has been referenced in Qml
        QString name;
        // objects in parse tree referencing the type
        QList<QmlParser::Object*> refObjects;
    };

    QmlScriptParser();
    ~QmlScriptParser();

    bool parse(const QByteArray &data, const QUrl &url = QUrl());

    QList<TypeReference*> referencedTypes() const;
    QList<QUrl> referencedResources() const;

    QmlParser::Object *tree() const;
    QList<Import> imports() const;

    void clear();

    QList<QmlError> errors() const;

// ### private:
    TypeReference *findOrCreateType(const QString &name);
    void setTree(QmlParser::Object *tree);

    void setScriptFile(const QString &filename) {_scriptFile = filename; }
    QString scriptFile() const { return _scriptFile; }

// ### private:
    QList<QmlError> _errors;

    QmlParser::Object *root;
    QList<Import> _imports;
    QList<TypeReference*> _refTypes;
    QList<QUrl> _refUrls;
    QString _scriptFile;
    QmlScriptParserJsASTData *data;
};

QT_END_NAMESPACE

QT_END_HEADER

#endif // QMLSCRIPTPARSER_P_H
